/*
   This program parses and then evaluates strings from Language-10.
*/

public class Examples
{
   public static void main(String[] args)
   {
      // Show that this language has static scope (final result should be 100).
      String expr1 = "( prog " +
                       "( define f1 ( lambda x 100 ) ) " +
                       "( define f2 ( lambda y ( begin " +
                                              "( var f1 ( lambda w 10 ) ) " +  // a local function
                                              "( apply f3 11 ) ) ) ) " +
                       "( define f3 ( lambda z ( apply f1 22 ) ) ) " +  // the global f1
                       "( apply f2 33 ) )";



      // recursive function
      String expr2 = "( prog " +
                       "( define even ( lambda n " +
                             "( && ( != n 1 ) " +       // use short-circuiting
                                  "( || ( == n 0 ) " +  // use short-circuiting
                                       "( apply even ( - n 2 ) ) ) ) ) ) " +
                       "( apply even 7 ) )";

      // mutually recursive functions
      String expr3 = "( prog " +
                       "( define even ( lambda n " +
                             "( || ( == n 0 ) " +       // use short-circuiting
                                  "( apply odd ( - n 1 ) ) ) ) ) " +
                       "( define odd ( lambda n " +
                            "( && ( >  n 0 ) " +        // use short-circuiting
                                 "( || ( == n 1 ) " +   // use short-circuiting
                                      "( apply even ( - n 1 ) ) ) ) ) ) " +
                       "( apply even 8 ) )";


      String expr = expr3;

      // build the abstract syntax tree that represents the expression
      Tree ast = BuildTree.buildTree( expr );

      // print the original expression
      System.out.println( expr + "\n" );
      // pretty-print the abstract syntax tree
      System.out.println( PrettyPrinter3.prettyPrint( ast ) + "\n" );
      // print the infix version of the expression
      System.out.println( AST2infix.ast2infix( ast ) + "\n" );

      try
      {
         // evaluate the expression (interpret the AST)
         Value value = Evaluate.evaluate( ast );

         System.out.println( "result = " + value );
      }
      catch (EvalException e)
      {
         e.printStackTrace();
      }
   }//main

}//Examples